package gamehost;

import jade.domain.FIPANames;
import jade.domain.FIPAAgentManagement.FailureException;
import jade.domain.FIPAAgentManagement.NotUnderstoodException;
import jade.domain.FIPAAgentManagement.RefuseException;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.proto.AchieveREResponder;
import strategy.onto.GamingOntology;

/**
 * @author  Christian Birmes
 * @version
 */
public class ReceivePlayer extends AchieveREResponder {
  protected AbsGameHost gameHost;

  /** Creates new ReceivePlayer */
  public ReceivePlayer(AbsGameHost gh) {
    super(gh, MessageTemplate.and(MessageTemplate.and(MessageTemplate.MatchPerformative(ACLMessage.REQUEST),
        MessageTemplate.MatchOntology(GamingOntology.ONTOLOGY_NAME)),
        MessageTemplate.and(MessageTemplate.MatchProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST),
        MessageTemplate.MatchContent("REGISTER"))));
    gameHost = gh;
  }

  protected ACLMessage prepareResponse(ACLMessage request) throws NotUnderstoodException, RefuseException {
    ACLMessage msg = request.createReply();
    msg.addReplyTo(gameHost.getAID());
    msg.setSender(gameHost.getAID());
    if (!gameHost.full() && !gameHost.isPlaying()) msg.setPerformative(ACLMessage.AGREE); else
      msg.setPerformative(ACLMessage.REFUSE);
    return msg;
  }

  protected ACLMessage prepareResultNotification(ACLMessage request, ACLMessage response) throws FailureException {
    ACLMessage msg = request.createReply();
    msg.addReplyTo(gameHost.getAID());
    msg.setSender(gameHost.getAID());
    if (gameHost.addPlayer(request.getSender())) {
      msg.setPerformative(ACLMessage.INFORM);
      msg.setContent(gameHost.getGame());
      if (gameHost.full()) gameHost.startPlaying();
    } else
      msg.setPerformative(ACLMessage.FAILURE);
    return msg;
  }
}
