package meiern;

import gamehost.AbsGameHost;
import gamehost.GameHostWindow;
import jade.core.AID;
import java.util.Random;
import java.util.Vector;
import strategy.Strategy;
import strategy.onto.StrategyDescription;

public class GameHost extends AbsGameHost {
  public static final int FIRST_TURN = -1;
  private int lastRealDice;
  private int lastBluffDice;
  private int lastLastDice;
  private AID lastPlayer;
  private static int[] dice = {31, 32, 41, 42, 43, 51, 52, 53, 54, 61, 62, 63, 64, 65, 11, 22, 33, 44, 55, 66, 21};
  private Vector points = new Vector();
  private final int START_POINTS = 3;
  private Random rnd = new Random();
  private GameHostWindow textWin;

  /** Creates new MeiernGameHost */
  public GameHost() {
    super();
    maxPlayers = 4;
    maxGames = 10000;
    game = "Meiern";
    fee = 6;
    prize = 15;
    textWin = new GameHostWindow(game);
    // ***** Fr jede Strategie eine Beschreibung erstellen *****
    makeMoveBehaviour = new Strategy[3];
    StrategyDescription sd = new StrategyDescription();
    sd.setGame(game);
    sd.setName("Test + 5");
    sd.setRisk(2);
    makeMoveBehaviour[0] = new Strategy(sd, new SendMoveT5(null));
    sd = new StrategyDescription();
    sd.setGame(game);
    sd.setName("Test + 2");
    sd.setRisk(1);
    makeMoveBehaviour[1] = new Strategy(sd, new SendMoveT2(null));
    sd = new StrategyDescription();
    sd.setGame(game);
    sd.setName("Bis 62");
    sd.setRisk(0);
    makeMoveBehaviour[2] = new Strategy(sd, new SendMoveBis62(null));
    evaluateMoveBehaviour = new EvaluateMove(this);
  }

  public void finishGame() {
    super.finishGame();
    points.removeAllElements();
  }

  public void resetGame() {
    super.resetGame();
    lastRealDice = FIRST_TURN;
    lastBluffDice = FIRST_TURN;
    lastPlayer = null;
  }

  public boolean addPlayer(AID p) {
    if (super.addPlayer(p)) {
      points.addElement(new Integer(START_POINTS));
      return true;
    } else
      return false;
  }

  public boolean deletePlayer(AID p) {
    int pos = players.indexOf(p);
    if (pos >= 0) points.removeElementAt(pos);

        /*ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
        msg.setSender(getAID());
        msg.addReceiver(p);
        msg.setContent("GameOver");
        send(msg);*/

    return super.deletePlayer(p);
  }

  public void incrementPlayerZaehler(AID loser) {
    if (loser == null) super.incrementPlayerZaehler(); else {
      decrementPoints(loser);
      //ende der Runde deshalb wieder von vorne
      lastRealDice = FIRST_TURN;
      if (players.size() == 1) {
        //nur noch einer da!
      } else {
        //noch sind also genug spieler da, deshalb geht das spiel weiter...
        if (players.indexOf(loser) == -1) playerZaehler = rnd.nextInt(players.size()); else
          playerZaehler = players.indexOf(loser);
      }
    }
  }

  //public AID getNextPlayer() {}
  public static int getDiceOfIndex(int index) { return dice[index]; }

  public static int getIndexOfDice(int d) {
    int i = 0;
    while ((i <= 20) && (dice[i] != d)) i++;
    return i;
  }

  public int getLastRealDice() { return lastRealDice; }

  public int getLastBluffDice() { return lastBluffDice; }

  public int getLastLastDice() { return lastLastDice; }

  public void setLastRealDice(int d) { lastRealDice = d; }

  public void setLastBluffDice(int d) { lastBluffDice = d; }

  public void setLastLastDice(int d) { lastLastDice = d; }

  public void decrementPoints(AID player) {
    int pos = players.indexOf(player);
    Integer i = (Integer)points.elementAt(pos);
    int newI = i.intValue() - 1;
    if (newI <= 0) {
      println(player.getLocalName() + " hat keine Punkte mehr und fliegt raus.");
      deletePlayer(player);
    } else {
      points.removeElementAt(pos);
      points.add(pos, new Integer(newI));
    }
  }

  public int getPoints(AID player) {
    int pos = players.indexOf(player);
    return ((Integer)points.elementAt(pos)).intValue();
  }

  public AID getLastPlayer() { return lastPlayer; }

  public void setLastPlayer(AID p) { lastPlayer = p; }

  public void printPoints() {
    println();
    println("*** Spielstand ***");
    for (int i = 0; i < players.size(); i++)
      println(((AID)players.elementAt(i)).getLocalName() + ": " + ((Integer)points.elementAt(i)).intValue());
    println();
  }

  public void print(String text) {
    textWin.print(text);
  }

  public void println(String text) {
    textWin.println(text);
  }

  public void println() {
    textWin.println();
  }
}
