package player;

import bank.onto.BankingOntology;
import jade.content.lang.sl.SLCodec;
import jade.core.AID;
import jade.core.behaviours.FSMBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPANames;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.gui.GuiAgent;
import jade.gui.GuiEvent;
import jade.lang.acl.ACLMessage;
import strategy.Strategy;
import strategy.onto.GamingOntology;
import strategy.onto.Lost;
import strategy.onto.Won;
import viergewinnt.C4GameHost;
import gamehost.AbsGameHost;
import jade.core.Agent;

/**
 * @author  Christian Birmes
 * @version
 */
public class Player extends GuiAgent {
  public static final int GAME_HOSTS_FOUND = 0;
  public static final int GAME_HOSTS_NOT_FOUND = 1;
  public static final int REGISTER_FAILURE = 2;
  public static final int REGISTER_OK = 3;
  public static final int NO_MORE_GAME_HOSTS = 4;
  //public static final int RULES_RECEIVED = 5;
  public static final int GAME_END = 6;
  public static final int MY_TURN = 7;
  public static final int MOVE_MADE = 8;
  public static final int MORE_GAME_HOSTS = 9;
  public static final int REGISTER_SENT = 10;
  private AID host = null; // Host, an dem Spieler angemeldet ist
  private DFAgentDescription[] hosts = null; // Liste mit allen Hosts
  private int currentHostNo = 0; // Host, bei dem gerade die Anmeldung versucht wird/gespielt wird
  private AID repository;
  private AID bank;
  private Strategy strategy;
  private int selectionStrategy = RANDOM;
  public final static int RANDOM = 0;
  public final static int BEST = 1;
  private String game;

  /** Creates new Player */
  public Player() {
    super();
  }

  protected void setup() {
    getContentManager().registerOntology(GamingOntology.getInstance());
    getContentManager().registerOntology(BankingOntology.getInstance());
    getContentManager().registerLanguage(new SLCodec());
    // beim DF registrieren
    DFAgentDescription dfad = new DFAgentDescription();
    dfad.addOntology(GamingOntology.ONTOLOGY_NAME);
    dfad.setName(getAID());
    try {
      DFService.register(this, dfad);
    } catch (FIPAException e) {
      System.err.println(getLocalName() + ": registration with DF failed. Killing myself.");
      doDelete();
    }
    //FSMBehaviour fsm = constructFSM();
    // Behaviour hinzufgen
    //addBehaviour(fsm);
    addBehaviour(new SearchGameHosts(this));
  }

  public void takeDown() {
    try {
      DFService.deregister(this);
    } catch (FIPAException e) {
      System.err.println(getLocalName() + ": error while deregistering.");
    }
  }

  public AID getHost() { return host; }

  public void setHost(AID h) { host = h; }

  public DFAgentDescription[] getHosts() { return hosts; }

  public void setHosts(DFAgentDescription[] dfad) { hosts = dfad; }

  public int getCurrentHostNo() { return currentHostNo; }

  public void setCurrentHostNo(int chn) { currentHostNo = chn; }

  public AID getCurrentHost() {
    return hosts[currentHostNo].getName();
  }

  public void incrementCurrentHostNo() { currentHostNo++; }

  private FSMBehaviour constructFSM() {
    // Zustnde einfgen
    FSMBehaviour fsm = new FSMBehaviour(this);
    fsm.registerFirstState(new SearchGameHosts(this), "SearchGameHosts");
    fsm.registerState(new CheckCredits(this), "MoreGameHosts");
    fsm.registerState(new RegisterWithGameHost(this), "Register");
    fsm.registerLastState(new Suicide(this), "End");
    // Transitionen einfgen
    fsm.registerTransition("SearchGameHosts", "End", GAME_HOSTS_NOT_FOUND);
    fsm.registerTransition("SearchGameHosts", "MoreGameHosts", GAME_HOSTS_FOUND);
    fsm.registerTransition("MoreGameHosts", "Register", MORE_GAME_HOSTS);
    fsm.registerTransition("MoreGameHosts", "End", NO_MORE_GAME_HOSTS);
    //fsm.registerTransition("Register", "End", REGISTER_OK);
    fsm.registerTransition("Register", "MoreGameHosts", REGISTER_FAILURE);
    return fsm;
  }

  public AID getRepository() {
    return repository;
  }

  public void setRepository(AID repository) {
    this.repository = repository;
  }

  public Strategy getStrategy() {
    return strategy;
  }

  public void setStrategy(Strategy strategy) {
    this.strategy = strategy;
  }

  public int getSelectionStrategy() {
    return selectionStrategy;
  }

  public void setSelectionStrategy(int selectionStrategy) {
    this.selectionStrategy = selectionStrategy;
  }

  public String getGame() {
    return game;
  }

  public void setGame(String game) {
    this.game = game;
  }

  public AID getBank() {
    return bank;
  }

  public void setBank(AID bank) {
    this.bank = bank;
  }

  public void sendWon() {
    ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
    msg.setOntology(GamingOntology.ONTOLOGY_NAME);
    msg.setLanguage(FIPANames.ContentLanguage.FIPA_SL);
    msg.addReceiver(repository);
    Won won = new Won();
    won.setStrategy(strategy.getDescription());
    try {
      getContentManager().fillContent(msg, won);
      send(msg);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  public void sendLost() {
    ACLMessage msg = new ACLMessage(ACLMessage.INFORM);
    msg.setOntology(GamingOntology.ONTOLOGY_NAME);
    msg.setLanguage(FIPANames.ContentLanguage.FIPA_SL);
    msg.addReceiver(repository);
    Lost lost = new Lost();
    lost.setStrategy(strategy.getDescription());
    try {
      getContentManager().fillContent(msg, lost);
      send(msg);
    } catch (Exception e) {
      e.printStackTrace();
    }
  }

  protected void onGuiEvent(GuiEvent guiEvent) {
  }
}
