package player;

import jade.core.behaviours.SimpleBehaviour;
import jade.domain.FIPANames;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.proto.AchieveREInitiator;
import strategy.onto.GamingOntology;

/**
 * @author  Christian Birmes
 * @version
 */
public class RegisterWithGameHost extends SimpleBehaviour {
  private int exitCode = Player.REGISTER_FAILURE;
  private boolean finished;
  private int state = 0;
  private AchieveREInitiator init;
  private SelectStrategy sel;
  private Player player;

  public RegisterWithGameHost(Player p) {
    super(p);
    player = p;
  }

  public void action() {
    switch (state) {
      case 0: {
          //System.out.println(myAgent.getLocalName() + " ist im RegisterWithGameHost-Verhalten");
          finished = false;
          state = 1;
          ACLMessage sendMsg = new ACLMessage(ACLMessage.REQUEST);
          sendMsg.setContent("REGISTER");
          sendMsg.setOntology(GamingOntology.ONTOLOGY_NAME);
          sendMsg.setProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST);
          sendMsg.setSender(myAgent.getAID());
          sendMsg.addReceiver(player.getCurrentHost());
          init = new AchieveREInitiator(myAgent, sendMsg) {
            protected void handleInform(ACLMessage response) {
              ((Player)myAgent).setHost(response.getSender());
              ((Player)myAgent).setGame(response.getContent());
            }
          };
          myAgent.addBehaviour(init);
          break;
        }
      case 1: {
          if (init.done()) state = 2; else
            block(100);
          break;
        }
      case 2: {
          if (player.getHost() == null) state = 4; else {
            sel = new SelectStrategy((Player)myAgent);
            myAgent.addBehaviour(sel);
            state = 3;
          }
          break;
        }
      case 3: {
          if (sel.done()) {
            state = 4;
            if (player.getStrategy() != null) {
              exitCode = Player.REGISTER_OK;
            }
          } else
            block(100);
          break;
        }
      case 4: {
          finished = true;
          if (exitCode == Player.REGISTER_OK) {
            System.out.println(myAgent.getLocalName() + ": Anmeldung bei " +
                player.getCurrentHost().getLocalName() + " erfolreich.");
            ACLMessage startMsg = myAgent.blockingReceive(MessageTemplate.and(MessageTemplate.MatchPerformative(ACLMessage.INFORM),
                MessageTemplate.and(MessageTemplate.MatchSender(player.getHost()), MessageTemplate.MatchContent("Start"))), 10000);
            if (startMsg == null) {
              System.out.println(myAgent.getLocalName() + ": Zeitberschreitung, Abmeldung von " +
                  player.getHost().getLocalName() + " erfolgt.");
              ACLMessage deregMsg = new ACLMessage(ACLMessage.INFORM);
              deregMsg.setSender(myAgent.getAID());
              deregMsg.setContent("Deregister");
              deregMsg.addReceiver(player.getHost());
              myAgent.send(deregMsg);
              myAgent.addBehaviour(new CheckCredits(myAgent));
            } else
              myAgent.addBehaviour(player.getStrategy().getBehaviour());
          } else {
            myAgent.addBehaviour(new CheckCredits(myAgent));
            System.out.println(myAgent.getLocalName() + ": Anmeldung bei " +
                player.getCurrentHost().getLocalName() + " fehlgeschlagen.");
          }
        }
    }
  }

  public boolean done() {
    return finished;
  }
}
