package player;

import bank.onto.BankingOntology;
import bank.onto.CreateAccount;
import jade.content.onto.basic.Action;
import jade.core.behaviours.OneShotBehaviour;
import jade.domain.DFService;
import jade.domain.FIPAException;
import jade.domain.FIPANames;
import jade.domain.FIPAAgentManagement.DFAgentDescription;
import jade.domain.FIPAAgentManagement.ServiceDescription;
import jade.lang.acl.ACLMessage;
import strategy.onto.GamingOntology;

/**
 * @author  Christian Birmes
 * @version
 */
public class SearchGameHosts extends OneShotBehaviour {
  protected Player myAgent;
  private int exitCode = player.Player.GAME_HOSTS_FOUND;

  /** Creates new SearchGameHost */
  public SearchGameHosts(Player player) {
    super(player);
    myAgent = player;
  }

  public void action() {
    //System.out.println(myAgent.getLocalName() + " ist im SearchGameHost-Verhalten");
    DFAgentDescription matchDFAD = new DFAgentDescription();
    ServiceDescription matchSD = new ServiceDescription();
    matchSD.addOntology(GamingOntology.ONTOLOGY_NAME);
    matchSD.setType("HOST");
    matchDFAD.addServices(matchSD);
    matchDFAD.addOntology(GamingOntology.ONTOLOGY_NAME);
    try {
      myAgent.setHosts(DFService.search(myAgent, matchDFAD));
    } catch (FIPAException e) {
      System.err.println(myAgent.getLocalName() + ": Suche beim DF fehlgeschlagen.");
      exitCode = Player.GAME_HOSTS_NOT_FOUND;
    }
    matchDFAD = new DFAgentDescription();
    matchSD = new ServiceDescription();
    matchSD.addOntology(GamingOntology.ONTOLOGY_NAME);
    matchSD.setType("REPOSITORY");
    matchDFAD.addServices(matchSD);
    matchDFAD.addOntology(GamingOntology.ONTOLOGY_NAME);
    try {
      myAgent.setRepository(DFService.search(myAgent, matchDFAD) [0].getName());
    } catch (FIPAException e) {
      System.err.println(myAgent.getLocalName() + ": Suche beim DF fehlgeschlagen.");
      exitCode = Player.GAME_HOSTS_NOT_FOUND;
    }
    matchDFAD = new DFAgentDescription();
    matchSD = new ServiceDescription();
    matchSD.addOntology(BankingOntology.ONTOLOGY_NAME);
    matchSD.setType("BANK");
    matchDFAD.addServices(matchSD);
    matchDFAD.addOntology(BankingOntology.ONTOLOGY_NAME);
    try {
      myAgent.setBank(DFService.search(myAgent, matchDFAD) [0].getName());
    } catch (FIPAException e) {
      System.err.println(myAgent.getLocalName() + ": Suche beim DF fehlgeschlagen.");
      exitCode = Player.GAME_HOSTS_NOT_FOUND;
    }
    ACLMessage msg = new ACLMessage(ACLMessage.REQUEST);
    msg.setLanguage(FIPANames.ContentLanguage.FIPA_SL);
    msg.setOntology(BankingOntology.ONTOLOGY_NAME);
    msg.addReceiver(myAgent.getBank());
    CreateAccount ca = new CreateAccount();
    ca.setOwner(myAgent.getAID());
    Action action = new Action();
    action.setActor(myAgent.getBank());
    action.setAction(ca);
    try {
      myAgent.getContentManager().fillContent(msg, action);
      myAgent.send(msg);
    } catch (Exception e) {
      e.printStackTrace();
      exitCode = Player.GAME_HOSTS_NOT_FOUND;
    }
    if (exitCode == Player.GAME_HOSTS_NOT_FOUND) myAgent.addBehaviour(new Suicide(myAgent)); else
      myAgent.addBehaviour(new CheckCredits(myAgent));
  }

    /*public int onEnd() {
        if (exitCode == GAME_HOSTS_NOT_FOUND) myAgent.addBehaviour(new Suicide(myAgent));
        else myAgent.abbBehaviour(new MoreGameHosts(myAgent));
        return exitCode;
    }*/
}
