package strategy;

import jade.content.Concept;
import jade.content.ContentElement;
import jade.content.onto.basic.Action;
import jade.domain.FIPANames;
import jade.lang.acl.ACLMessage;
import jade.lang.acl.MessageTemplate;
import jade.proto.AchieveREResponder;
import jade.util.leap.ArrayList;
import java.io.IOException;
import strategy.onto.GamingOntology;
import strategy.onto.GetMostSuccessfullStrategy;
import strategy.onto.GetStrategies;
import strategy.onto.GetStrategiesByRisk;
import strategy.onto.GetStrategy;
import strategy.onto.Strategies;
import strategy.onto.StrategyDescription;

/* Generated by Together */

public class HandleRequests extends AchieveREResponder {
  private StrategyRepository sr;

  public HandleRequests(StrategyRepository r) {
    super(r, MessageTemplate.and(MessageTemplate.MatchLanguage(FIPANames.ContentLanguage.FIPA_SL),
        MessageTemplate.and(MessageTemplate.MatchOntology(GamingOntology.ONTOLOGY_NAME),
        MessageTemplate.and(MessageTemplate.MatchProtocol(FIPANames.InteractionProtocol.FIPA_REQUEST),
        MessageTemplate.MatchPerformative(ACLMessage.REQUEST)))));
    sr = r;
  }

  protected ACLMessage prepareResponse(ACLMessage request) {
    ContentElement ce = null;
    Concept concept = null;
    ACLMessage response = request.createReply();
    try {
      ce = myAgent.getContentManager().extractContent(request);
    } catch (Exception e) {
      response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
    }
    if (ce instanceof Action) {
      concept = ((Action)ce).getAction();
      if (concept instanceof GetStrategy) {
        StrategyDescription desc = new StrategyDescription();
        if (sr.getStrategy(((GetStrategy)concept).getStrategy()) != null)
          response.setPerformative(ACLMessage.AGREE); else
          response.setPerformative(ACLMessage.REFUSE);
      } else if (concept instanceof GetStrategiesByRisk) {
        GetStrategiesByRisk gs = (GetStrategiesByRisk)concept;
        if (sr.getStrategiesByRisk(gs.getGame(), gs.getRisk()).length > 0)
          response.setPerformative(ACLMessage.AGREE); else
          response.setPerformative(ACLMessage.REFUSE);
      } else if (concept instanceof GetMostSuccessfullStrategy) {
        if (sr.getMostSuccessfullStrategy(((GetMostSuccessfullStrategy)concept).getGame()) != null)
          response.setPerformative(ACLMessage.AGREE); else
          response.setPerformative(ACLMessage.REFUSE);
      } else if (concept instanceof GetStrategies) {
        if (sr.getStrategies(((GetStrategies)concept).getGame()).length > 0)
          response.setPerformative(ACLMessage.AGREE); else
          response.setPerformative(ACLMessage.REFUSE);
      } else
        response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
    } else
      response.setPerformative(ACLMessage.NOT_UNDERSTOOD);
    return response;
  }

  protected ACLMessage prepareResultNotification(ACLMessage request, ACLMessage myResponse) {
    ContentElement ce = null;
    Concept concept = null;
    ACLMessage response = request.createReply();
    try {
      ce = myAgent.getContentManager().extractContent(request);
    } catch (Exception e) {
      response.setPerformative(ACLMessage.FAILURE);
    }
    if (ce instanceof Action) {
      concept = ((Action)ce).getAction();
      if (concept instanceof GetStrategy) {
        StrategyDescription desc;
        try {
          GetStrategy gs = (GetStrategy)concept;
          desc = gs.getStrategy();
          response.setContentObject(sr.getStrategy(desc));
          response.setPerformative(ACLMessage.INFORM);
          ((StrategyRepository)myAgent).addUse(desc.getGame(), desc.getName());
        } catch (IOException e) {
          response.setPerformative(ACLMessage.FAILURE);
        }
      } else if (concept instanceof GetStrategiesByRisk) {
        GetStrategiesByRisk gs = (GetStrategiesByRisk)concept;
        StrategyDescription[] sd = sr.getStrategiesByRisk(gs.getGame(), gs.getRisk());
        ArrayList l = new ArrayList();
        for (int i = 0; i < sd.length; i++) l.add(sd[i]);
        Strategies s = new Strategies();
        s.setStrategies(l);
        try {
          myAgent.getContentManager().fillContent(response, s);
          response.setPerformative(ACLMessage.INFORM);
        } catch (Exception e) {
          response.setPerformative(ACLMessage.FAILURE);
        }
      } else if (concept instanceof GetMostSuccessfullStrategy) {
        Strategy st = sr.getMostSuccessfullStrategy(((GetMostSuccessfullStrategy)concept).getGame());
        StrategyDescription sd = st.getDescription();
        try {
          response.setContentObject(st);
          response.setPerformative(ACLMessage.INFORM);
          ((StrategyRepository)myAgent).addUse(sd.getGame(), sd.getName());
        } catch (IOException e) {
          response.setPerformative(ACLMessage.FAILURE);
        }
      } else if (concept instanceof GetStrategies) {
        StrategyDescription[] sd = sr.getStrategies(((GetStrategies)concept).getGame());
        ArrayList l = new ArrayList();
        for (int i = 0; i < sd.length; i++) l.add(sd[i]);
        Strategies s = new Strategies();
        s.setStrategies(l);
        try {
          myAgent.getContentManager().fillContent(response, s);
          response.setPerformative(ACLMessage.INFORM);
        } catch (Exception e) {
          response.setPerformative(ACLMessage.FAILURE);
        }
      } else
        response.setPerformative(ACLMessage.FAILURE);
    } else
      response.setPerformative(ACLMessage.FAILURE);
    return response;
  }
}
